###NOTE: This test module requires use of credentials. The first run through of the tests will prompt for credentials from the logged on user.

Import-Module $PSScriptRoot\..\DSCResources\MSFT_xExchAntiMalwareScanning\MSFT_xExchAntiMalwareScanning.psm1
Import-Module $PSScriptRoot\..\Misc\xExchangeCommon.psm1 -Verbose:0
Import-Module $PSScriptRoot\xExchange.Tests.Common.psm1 -Verbose:0

#Check if Exchange is installed on this machine. If not, we can't run tests
[bool]$exchangeInstalled = IsSetupComplete

if ($exchangeInstalled)
{
    #Get required credentials to use for the test
    if ($null -eq $Global:ShellCredentials)
    {
        [PSCredential]$Global:ShellCredentials = Get-Credential -Message "Enter credentials for connecting a Remote PowerShell session to Exchange"
    }

    Describe "Test Enabling and Disabling Malware Scanning" {
        #Test enabling with no restart of the service
        $testParams = @{
            Enabled = $true
            Credential = $Global:ShellCredentials
            AllowServiceRestart = $false
        }

        $expectedGetResults = @{
            Enabled = $true
        }

        Test-AllTargetResourceFunctions -Params $testParams -ContextLabel "Enable Malware Scanning - No Restart" -ExpectedGetResults $expectedGetResults
        

        #Test disabling with no restart of the service
        $testParams.Enabled = $false
        $expectedGetResults.Enabled = $false

        Test-AllTargetResourceFunctions -Params $testParams -ContextLabel "Disable Malware Scanning - No Restart" -ExpectedGetResults $expectedGetResults


        #Test enabling with a restart of the service
        $testParams.Enabled = $true
        $testParams.AllowServiceRestart = $true
        $expectedGetResults.Enabled = $true

        Test-AllTargetResourceFunctions -Params $testParams -ContextLabel "Enable Malware Scanning - With Restart" -ExpectedGetResults $expectedGetResults


        #Test disabling with a restart of the service
        $testParams.Enabled = $false
        $expectedGetResults.Enabled = $false

        Test-AllTargetResourceFunctions -Params $testParams -ContextLabel "Disable Malware Scanning - With Restart" -ExpectedGetResults $expectedGetResults
    }
}
else
{
    Write-Verbose "Tests in this file require that Exchange is installed to be run."
}
    
